@@ -53,7 +53,36 @@ def to_host_port(host, default=AMQP_PORT):
 
 
 class _AbstractTransport:
-    """Common superclass for TCP and SSL transports."""
+    """Common superclass for TCP and SSL transports.
+
+    PARAMETERS:
+        host: str
+
+            Broker address in format ``HOSTNAME:PORT``.
+
+        connect_timeout: int
+
+            Timeout of creating new connection.
+
+        read_timeout: int
+
+            sets ``SO_RCVTIMEO`` parameter of socket.
+
+        write_timeout: int
+
+            sets ``SO_SNDTIMEO`` parameter of socket.
+
+        socket_settings: dict
+
+            dictionary containing `optname` and ``optval`` passed to
+            ``setsockopt(2)``.
+
+        raise_on_initial_eintr: bool
+
+            when True, ``socket.timeout`` is raised
+            when exception is received during first read. See ``_read()`` for
+            details.
+    """
 
     def __init__(self, host, connect_timeout=None,
                  read_timeout=None, write_timeout=None,
@@ -68,6 +97,14 @@ class _AbstractTransport:
         self.write_timeout = write_timeout
         self.socket_settings = socket_settings
 
+    def __repr__(self):
+        if self.sock:
+            src = f'{self.sock.getsockname()[0]}:{self.sock.getsockname()[1]}'
+            dst = f'{self.sock.getpeername()[0]}:{self.sock.getpeername()[1]}'
+            return f'<{type(self).__name__}: {src} -> {dst} at {id(self):#x}>'
+        else:
+            return f'<{type(self).__name__}: (disconnected) at {id(self):#x}>'
+
     def connect(self):
         try:
             # are we already connected?
@@ -219,7 +256,7 @@ class _AbstractTransport:
 
     def _read(self, n, initial=False):
         """Read exactly n bytes from the peer."""
-        raise NotImplementedError('Must be overriden in subclass')
+        raise NotImplementedError('Must be overridden in subclass')
 
     def _setup_transport(self):
         """Do any additional initialization of the class."""
@@ -231,7 +268,7 @@ class _AbstractTransport:
 
     def _write(self, s):
         """Completely write a string to the peer."""
-        raise NotImplementedError('Must be overriden in subclass')
+        raise NotImplementedError('Must be overridden in subclass')
 
     def close(self):
         if self.sock is not None:
@@ -239,12 +276,26 @@ class _AbstractTransport:
             # Call shutdown first to make sure that pending messages
             # reach the AMQP broker if the program exits after
             # calling this method.
-            self.sock.shutdown(socket.SHUT_RDWR)
+            try:
+                self.sock.shutdown(socket.SHUT_RDWR)
+            except OSError:
+                pass
             self.sock.close()
             self.sock = None
         self.connected = False
 
     def read_frame(self, unpack=unpack):
+        """Parse AMQP frame.
+
+        Frame has following format::
+
+            0      1         3         7                   size+7      size+8
+            +------+---------+---------+   +-------------+   +-----------+
+            | type | channel |  size   |   |   payload   |   | frame-end |
+            +------+---------+---------+   +-------------+   +-----------+
+             octet    short     long        'size' octets        octet
+
+        """
         read = self._read
         read_frame_buffer = EMPTY_BUFFER
         try:
@@ -268,7 +319,7 @@ class _AbstractTransport:
             else:
                 payload = read(size)
             read_frame_buffer += payload
-            ch = ord(read(1))
+            frame_end = ord(read(1))
         except socket.timeout:
             self._read_buffer = read_frame_buffer + self._read_buffer
             raise
@@ -292,11 +343,12 @@ class _AbstractTransport:
             if exc.errno not in _UNAVAIL:
                 self.connected = False
             raise
-        if ch == 206:  # '\xce'
+        # frame-end octet must contain '\xce' value
+        if frame_end == 206:
             return frame_type, channel, payload
         else:
             raise UnexpectedFrame(
-                f'Received {ch:#04x} while expecting 0xce')
+                f'Received frame_end {frame_end:#04x} while expecting 0xce')
 
     def write(self, s):
         try:
@@ -310,7 +362,39 @@ class _AbstractTransport:
 
 
 class SSLTransport(_AbstractTransport):
-    """Transport that works over SSL."""
+    """Transport that works over SSL.
+
+    PARAMETERS:
+        host: str
+
+            Broker address in format ``HOSTNAME:PORT``.
+
+        connect_timeout: int
+
+            Timeout of creating new connection.
+
+        ssl: bool|dict
+
+            parameters of TLS subsystem.
+                - when ``ssl`` is not dictionary, defaults of TLS are used
+                - otherwise:
+                    - if ``ssl`` dictionary contains ``context`` key,
+                      :attr:`~SSLTransport._wrap_context` is used for wrapping
+                      socket. ``context`` is a dictionary passed to
+                      :attr:`~SSLTransport._wrap_context` as context parameter.
+                      All others items from ``ssl`` argument are passed as
+                      ``sslopts``.
+                    - if ``ssl`` dictionary does not contain ``context`` key,
+                      :attr:`~SSLTransport._wrap_socket_sni` is used for
+                      wrapping socket. All items in ``ssl`` argument are
+                      passed to :attr:`~SSLTransport._wrap_socket_sni` as
+                      parameters.
+
+        kwargs:
+
+            additional arguments of
+            :class:`~amqp.transport._AbstractTransport` class
+    """
 
     def __init__(self, host, connect_timeout=None, ssl=None, **kwargs):
         self.sslopts = ssl if isinstance(ssl, dict) else {}
@@ -330,19 +414,96 @@ class SSLTransport(_AbstractTransport):
         return self._wrap_socket_sni(sock, **sslopts)
 
     def _wrap_context(self, sock, sslopts, check_hostname=None, **ctx_options):
+        """Wrap socket without SNI headers.
+
+        PARAMETERS:
+            sock: socket.socket
+
+            Socket to be wrapped.
+
+            sslopts: dict
+
+                Parameters of  :attr:`ssl.SSLContext.wrap_socket`.
+
+            check_hostname
+
+                Whether to match the peer cert’s hostname. See
+                :attr:`ssl.SSLContext.check_hostname` for details.
+
+            ctx_options
+
+                Parameters of :attr:`ssl.create_default_context`.
+        """
         ctx = ssl.create_default_context(**ctx_options)
         ctx.check_hostname = check_hostname
         return ctx.wrap_socket(sock, **sslopts)
 
     def _wrap_socket_sni(self, sock, keyfile=None, certfile=None,
-                         server_side=False, cert_reqs=ssl.CERT_NONE,
-                         do_handshake_on_connect=False,
+                         server_side=False, cert_reqs=None,
+                         ca_certs=None, do_handshake_on_connect=False,
                          suppress_ragged_eofs=True, server_hostname=None,
-                         ssl_version=ssl.PROTOCOL_TLS):
+                         ciphers=None, ssl_version=None):
         """Socket wrap with SNI headers.
 
-        stdlib `ssl.SSLContext.wrap_socket` method augmented with support for
-        setting the server_hostname field required for SNI hostname header
+        stdlib :attr:`ssl.SSLContext.wrap_socket` method augmented with support
+        for setting the server_hostname field required for SNI hostname header.
+
+        PARAMETERS:
+            sock: socket.socket
+
+                Socket to be wrapped.
+
+            keyfile: str
+
+                Path to the private key
+
+            certfile: str
+
+                Path to the certificate
+
+            server_side: bool
+
+                Identifies whether server-side or client-side
+                behavior is desired from this socket. See
+                :attr:`~ssl.SSLContext.wrap_socket` for details.
+
+            cert_reqs: ssl.VerifyMode
+
+                When set to other than :attr:`ssl.CERT_NONE`, peers certificate
+                is checked. Possible values are :attr:`ssl.CERT_NONE`,
+                :attr:`ssl.CERT_OPTIONAL` and :attr:`ssl.CERT_REQUIRED`.
+
+            ca_certs: str
+
+                Path to “certification authority” (CA) certificates
+                used to validate other peers’ certificates when ``cert_reqs``
+                is other than :attr:`ssl.CERT_NONE`.
+
+            do_handshake_on_connect: bool
+
+                Specifies whether to do the SSL
+                handshake automatically. See
+                :attr:`~ssl.SSLContext.wrap_socket` for details.
+
+            suppress_ragged_eofs (bool):
+
+                See :attr:`~ssl.SSLContext.wrap_socket` for details.
+
+            server_hostname: str
+
+                Specifies the hostname of the service which
+                we are connecting to. See :attr:`~ssl.SSLContext.wrap_socket`
+                for details.
+
+            ciphers: str
+
+                Available ciphers for sockets created with this
+                context. See :attr:`ssl.SSLContext.set_ciphers`
+
+            ssl_version:
+
+                Protocol of the SSL Context. The value is one of
+                ``ssl.PROTOCOL_*`` constants.
         """
         opts = {
             'sock': sock,
@@ -352,16 +513,48 @@ class SSLTransport(_AbstractTransport):
             'server_hostname': server_hostname,
         }
 
+        if ssl_version is None:
+            ssl_version = (
+                ssl.PROTOCOL_TLS_SERVER
+                if server_side
+                else ssl.PROTOCOL_TLS_CLIENT
+            )
+
         context = ssl.SSLContext(ssl_version)
+
         if certfile is not None:
             context.load_cert_chain(certfile, keyfile)
-        if cert_reqs != ssl.CERT_NONE:
-            context.check_hostname = True
-        # Set SNI headers if supported
-        if (server_hostname is not None) and (
-                hasattr(ssl, 'HAS_SNI') and ssl.HAS_SNI) and (
-                hasattr(ssl, 'SSLContext')):
+        if ca_certs is not None:
+            context.load_verify_locations(ca_certs)
+        if ciphers is not None:
+            context.set_ciphers(ciphers)
+        # Set SNI headers if supported.
+        # Must set context.check_hostname before setting context.verify_mode
+        # to avoid setting context.verify_mode=ssl.CERT_NONE while
+        # context.check_hostname is still True (the default value in context
+        # if client-side) which results in the following exception:
+        # ValueError: Cannot set verify_mode to CERT_NONE when check_hostname
+        # is enabled.
+        try:
+            context.check_hostname = (
+                ssl.HAS_SNI and server_hostname is not None
+            )
+        except AttributeError:
+            pass  # ask forgiveness not permission
+
+        # See note above re: ordering for context.check_hostname and
+        # context.verify_mode assignments.
+        if cert_reqs is not None:
             context.verify_mode = cert_reqs
+
+        if ca_certs is None and context.verify_mode != ssl.CERT_NONE:
+            purpose = (
+                ssl.Purpose.CLIENT_AUTH
+                if server_side
+                else ssl.Purpose.SERVER_AUTH
+            )
+            context.load_default_certs(purpose)
+
         sock = context.wrap_socket(**opts)
         return sock
 
@@ -416,7 +609,10 @@ class SSLTransport(_AbstractTransport):
 
 
 class TCPTransport(_AbstractTransport):
-    """Transport that deals directly with TCP socket."""
+    """Transport that deals directly with TCP socket.
+
+    All parameters are :class:`~amqp.transport._AbstractTransport` class.
+    """
 
     def _setup_transport(self):
         # Setup to _write() directly to the socket, and
@@ -454,7 +650,29 @@ def Transport(host, connect_timeout=None, ssl=False, **kwargs):
     """Create transport.
 
     Given a few parameters from the Connection constructor,
-    select and create a subclass of _AbstractTransport.
+    select and create a subclass of
+    :class:`~amqp.transport._AbstractTransport`.
+
+    PARAMETERS:
+
+        host: str
+
+            Broker address in format ``HOSTNAME:PORT``.
+
+        connect_timeout: int
+
+            Timeout of creating new connection.
+
+        ssl: bool|dict
+
+            If set, :class:`~amqp.transport.SSLTransport` is used
+            and ``ssl`` parameter is passed to it. Otherwise
+            :class:`~amqp.transport.TCPTransport` is used.
+
+        kwargs:
+
+            additional arguments of :class:`~amqp.transport._AbstractTransport`
+            class
     """
     transport = SSLTransport if ssl else TCPTransport
     return transport(host, connect_timeout=connect_timeout, ssl=ssl, **kwargs)
