@@ -1,9 +1,9 @@
+import re
 import socket
 import warnings
 from unittest.mock import Mock, call, patch
 
 import pytest
-from case import ContextMock
 
 from amqp import Connection, spec
 from amqp.connection import SSLError
@@ -12,6 +12,8 @@ from amqp.exceptions import (ConnectionError, NotFound,
 from amqp.sasl import AMQPLAIN, EXTERNAL, GSSAPI, PLAIN, SASL
 from amqp.transport import TCPTransport
 
+from t.mocks import ContextMock
+
 
 class test_Connection:
 
@@ -321,10 +323,17 @@ class test_Connection:
                 channel.collect.assert_called_with()
         assert self.conn._transport is None
 
-    def test_collect__channel_raises_socket_error(self):
-        self.conn.channels = self.conn.channels = {1: Mock(name='c1')}
-        self.conn.channels[1].collect.side_effect = socket.error()
+    def test_collect__transport_socket_raises_os_error(self):
+        self.conn.transport = TCPTransport('localhost:5672')
+        sock = self.conn.transport.sock = Mock(name='sock')
+        channel = Mock(name='c1')
+        self.conn.channels = {1: channel}
+        sock.shutdown.side_effect = OSError
         self.conn.collect()
+        channel.collect.assert_called_with()
+        sock.close.assert_called_with()
+        assert self.conn._transport is None
+        assert self.conn.channels is None
 
     def test_collect_no_transport(self):
         self.conn = Connection()
@@ -501,3 +510,35 @@ class test_Connection:
     def test_server_capabilities(self):
         self.conn.server_properties['capabilities'] = {'foo': 1}
         assert self.conn.server_capabilities == {'foo': 1}
+
+    @pytest.mark.parametrize(
+        'conn_kwargs,expected_vhost', [
+            ({}, '/'),
+            ({'user_id': 'test_user', 'password': 'test_pass'}, '/'),
+            ({'virtual_host': 'test_vhost'}, 'test_vhost')
+        ]
+    )
+    def test_repr_disconnected(self, conn_kwargs, expected_vhost):
+        assert re.fullmatch(
+            r'<AMQP Connection: broker.com:1234/{} '
+            r'\(disconnected\) at 0x.*>'.format(expected_vhost),
+            repr(Connection(host='broker.com:1234', **conn_kwargs))
+        )
+
+    @pytest.mark.parametrize(
+        'conn_kwargs,expected_vhost', [
+            ({}, '/'),
+            ({'user_id': 'test_user', 'password': 'test_pass'}, '/'),
+            ({'virtual_host': 'test_vhost'}, 'test_vhost')
+        ]
+    )
+    def test_repr_connected(self, conn_kwargs, expected_vhost):
+        c = Connection(host='broker.com:1234', **conn_kwargs)
+        c._transport = Mock(name='transport')
+        assert re.fullmatch(
+            r'<AMQP Connection: broker.com:1234/{} using {} at 0x.*>'.format(
+                expected_vhost,
+                repr(c.transport)
+            ),
+            repr(c)
+        )
